@@ -2,7 +2,7 @@
  *
  *  Wireless daemon for Linux
  *
- *  Copyright (C) 2013-2014  Intel Corporation. All rights reserved.
+ *  Copyright (C) 2013-2019  Intel Corporation. All rights reserved.
  *
  *  This library is free software; you can redistribute it and/or
  *  modify it under the terms of the GNU Lesser General Public
@@ -29,14 +29,15 @@
 #define IWD_DEVICE_INTERFACE "net.connman.iwd.Device"
 #define IWD_NETWORK_INTERFACE "net.connman.iwd.Network"
 #define IWD_AGENT_INTERFACE "net.connman.iwd.Agent"
-#define IWD_WSC_INTERFACE "net.connman.iwd.WiFiSimpleConfiguration"
+#define IWD_WSC_INTERFACE "net.connman.iwd.SimpleConfiguration"
 #define IWD_KNOWN_NETWORK_INTERFACE "net.connman.iwd.KnownNetwork"
 #define IWD_SIGNAL_AGENT_INTERFACE "net.connman.iwd.SignalLevelAgent"
 #define IWD_AP_INTERFACE "net.connman.iwd.AccessPoint"
 #define IWD_ADHOC_INTERFACE "net.connman.iwd.AdHoc"
 #define IWD_STATION_INTERFACE "net.connman.iwd.Station"
 
-#define IWD_AGENT_MANAGER_PATH "/"
+#define IWD_BASE_PATH "/net/connman/iwd"
+#define IWD_AGENT_MANAGER_PATH IWD_BASE_PATH
 
 struct l_dbus;
 
