@@ -2,7 +2,7 @@
  *
  *  Wireless daemon for Linux
  *
- *  Copyright (C) 2017  Intel Corporation. All rights reserved.
+ *  Copyright (C) 2017-2019  Intel Corporation. All rights reserved.
  *
  *  This library is free software; you can redistribute it and/or
  *  modify it under the terms of the GNU Lesser General Public
@@ -55,7 +55,8 @@ static bool ft_calculate_fte_mic(struct handshake_state *hs, uint8_t seq_num,
 	int iov_elems = 0;
 	struct l_checksum *checksum;
 	const uint8_t *kck = handshake_state_get_kck(hs);
-	uint8_t zero_mic[16] = {};
+	size_t kck_len = handshake_state_get_kck_len(hs);
+	uint8_t zero_mic[24] = {};
 
 	iov[iov_elems].iov_base = hs->spa;
 	iov[iov_elems++].iov_len = 6;
@@ -79,10 +80,10 @@ static bool ft_calculate_fte_mic(struct handshake_state *hs, uint8_t seq_num,
 		iov[iov_elems++].iov_len = 4;
 
 		iov[iov_elems].iov_base = zero_mic;
-		iov[iov_elems++].iov_len = 16;
+		iov[iov_elems++].iov_len = kck_len;
 
-		iov[iov_elems].iov_base = (void *) (fte + 20);
-		iov[iov_elems++].iov_len = fte[1] + 2 - 20;
+		iov[iov_elems].iov_base = (void *) (fte + 4 + kck_len);
+		iov[iov_elems++].iov_len = fte[1] + 2 - 4 - kck_len;
 	}
 
 	if (ric) {
@@ -90,12 +91,16 @@ static bool ft_calculate_fte_mic(struct handshake_state *hs, uint8_t seq_num,
 		iov[iov_elems++].iov_len = ric[1] + 2;
 	}
 
-	checksum = l_checksum_new_cmac_aes(kck, 16);
+	if (kck_len == 16)
+		checksum = l_checksum_new_cmac_aes(kck, kck_len);
+	else
+		checksum = l_checksum_new_hmac(L_CHECKSUM_SHA384, kck, kck_len);
+
 	if (!checksum)
 		return false;
 
 	l_checksum_updatev(checksum, iov, iov_elems);
-	l_checksum_get_digest(checksum, out_mic, 16);
+	l_checksum_get_digest(checksum, out_mic, kck_len);
 	l_checksum_free(checksum);
 
 	return true;
@@ -227,7 +232,7 @@ static bool ft_parse_associate_resp_frame(const uint8_t *frame, size_t frame_len
 		}
 	}
 
-	*out_status = body->status_code;
+	*out_status = L_LE16_TO_CPU(body->status_code);
 
 	return true;
 }
@@ -237,6 +242,7 @@ static int ft_tx_reassociate(struct ft_sm *ft)
 	struct iovec iov[3];
 	int iov_elems = 0;
 	struct handshake_state *hs = ft->hs;
+	uint32_t kck_len = handshake_state_get_kck_len(hs);
 	bool is_rsn = hs->supplicant_ie != NULL;
 	uint8_t *rsne = NULL;
 
@@ -302,13 +308,13 @@ static int ft_tx_reassociate(struct ft_sm *ft)
 		memcpy(ft_info.snonce, hs->snonce, 32);
 
 		fte = alloca(256);
-		ie_build_fast_bss_transition(&ft_info, fte);
+		ie_build_fast_bss_transition(&ft_info, kck_len, fte);
 
 		if (!ft_calculate_fte_mic(hs, 5, rsne, fte, NULL, ft_info.mic))
 			goto error;
 
 		/* Rebuild the FT IE now with the MIC included */
-		ie_build_fast_bss_transition(&ft_info, fte);
+		ie_build_fast_bss_transition(&ft_info, kck_len, fte);
 
 		iov[iov_elems].iov_base = fte;
 		iov[iov_elems].iov_len = fte[1] + 2;
@@ -330,6 +336,7 @@ static int ft_process_ies(struct ft_sm *ft, const uint8_t *ies, size_t ies_len)
 	const uint8_t *mde = NULL;
 	const uint8_t *fte = NULL;
 	struct handshake_state *hs = ft->hs;
+	uint32_t kck_len = handshake_state_get_kck_len(hs);
 	bool is_rsn;
 
 	/* Check 802.11r IEs */
@@ -428,17 +435,17 @@ static int ft_process_ies(struct ft_sm *ft, const uint8_t *ies, size_t ies_len)
 	 */
 	if (is_rsn) {
 		struct ie_ft_info ft_info;
-		uint8_t zeros[16] = {};
+		uint8_t zeros[24] = {};
 
 		if (!fte)
 			goto ft_error;
 
 		if (ie_parse_fast_bss_transition_from_data(fte, fte[1] + 2,
-								&ft_info) < 0)
+						kck_len, &ft_info) < 0)
 			goto ft_error;
 
 		if (ft_info.mic_element_count != 0 ||
-				memcmp(ft_info.mic, zeros, 16))
+				memcmp(ft_info.mic, zeros, kck_len))
 			goto ft_error;
 
 		if (hs->r0khid_len != ft_info.r0khid_len ||
@@ -524,6 +531,7 @@ static int ft_rx_associate(struct auth_proto *ap, const uint8_t *frame,
 {
 	struct ft_sm *ft = l_container_of(ap, struct ft_sm, ap);
 	struct handshake_state *hs = ft->hs;
+	uint32_t kck_len = handshake_state_get_kck_len(hs);
 	const uint8_t *rsne = NULL;
 	const uint8_t *mde = NULL;
 	const uint8_t *fte = NULL;
@@ -586,10 +594,10 @@ static int ft_rx_associate(struct auth_proto *ap, const uint8_t *frame,
 
 	if (fte) {
 		struct ie_ft_info ft_info;
-		uint8_t mic[16];
+		uint8_t mic[24];
 
 		if (ie_parse_fast_bss_transition_from_data(fte, fte[1] + 2,
-								&ft_info) < 0)
+						kck_len, &ft_info) < 0)
 			return -EBADMSG;
 
 		/*
@@ -603,7 +611,7 @@ static int ft_rx_associate(struct auth_proto *ap, const uint8_t *frame,
 			return -EBADMSG;
 
 		if (ft_info.mic_element_count != 3 ||
-				memcmp(ft_info.mic, mic, 16))
+				memcmp(ft_info.mic, mic, kck_len))
 			return -EBADMSG;
 
 		if (hs->r0khid_len != ft_info.r0khid_len ||
@@ -665,6 +673,7 @@ static bool ft_start(struct auth_proto *ap)
 {
 	struct ft_sm *ft = l_container_of(ap, struct ft_sm, ap);
 	struct handshake_state *hs = ft->hs;
+	uint32_t kck_len = handshake_state_get_kck_len(hs);
 	bool is_rsn = hs->supplicant_ie != NULL;
 	uint8_t mde[5];
 	struct iovec iov[3];
@@ -732,7 +741,7 @@ static bool ft_start(struct auth_proto *ap)
 		memcpy(ft_info.snonce, hs->snonce, 32);
 
 		fte = alloca(256);
-		ie_build_fast_bss_transition(&ft_info, fte);
+		ie_build_fast_bss_transition(&ft_info, kck_len, fte);
 
 		iov[iov_elems].iov_base = fte;
 		iov[iov_elems].iov_len = fte[1] + 2;
