@@ -145,8 +145,8 @@ static bool handshake_state_setup_own_ciphers(struct handshake_state *s,
 	s->group_management_cipher = info->group_management_cipher;
 
 	/*
-	 * Dont set MFP for OSEN otherwise EAPoL will attempt to negotiate a
-	 * iGTK which is not allowe for OSEN.
+	 * Don't set MFP for OSEN otherwise EAPoL will attempt to negotiate a
+	 * iGTK which is not allowed for OSEN.
 	 */
 	if (!s->osen_ie)
 		s->mfp = info->mfpc;
@@ -558,7 +558,7 @@ void handshake_state_install_ptk(struct handshake_state *s)
 		uint32_t cipher = ie_rsn_cipher_suite_to_cipher(
 							s->pairwise_cipher);
 
-		handshake_event(s, HANDSHAKE_EVENT_SETTING_KEYS, NULL);
+		handshake_event(s, HANDSHAKE_EVENT_SETTING_KEYS);
 
 		install_tk(s, handshake_get_tk(s), cipher);
 	}
@@ -912,10 +912,3 @@ bool handshake_decode_fte_key(struct handshake_state *s, const uint8_t *wrapped,
 
 	return true;
 }
-
-void handshake_event(struct handshake_state *hs,
-			enum handshake_event event, void *event_data)
-{
-	if (hs->event_func)
-		hs->event_func(hs, event, event_data, hs->user_data);
-}
