@@ -2,7 +2,7 @@
  *
  *  Wireless daemon for Linux
  *
- *  Copyright (C) 2013-2015  Intel Corporation. All rights reserved.
+ *  Copyright (C) 2013-2019  Intel Corporation. All rights reserved.
  *
  *  This library is free software; you can redistribute it and/or
  *  modify it under the terms of the GNU Lesser General Public
@@ -33,6 +33,7 @@
 #include <ell/ell.h>
 
 #include "src/missing.h"
+#include "src/module.h"
 #include "src/ie.h"
 #include "src/crypto.h"
 #include "src/iwd.h"
@@ -551,7 +552,7 @@ int network_autoconnect(struct network *network, struct scan_bss *bss)
 
 	if (!wiphy_select_cipher(wiphy, rsn.pairwise_ciphers) ||
 			!wiphy_select_cipher(wiphy, rsn.group_cipher)) {
-		l_debug("Cipher mis-match");
+		l_debug("Cipher mismatch");
 		ret = -ENETUNREACH;
 		goto close_settings;
 	}
@@ -1376,6 +1377,8 @@ void network_rank_update(struct network *network, bool connected)
 	if (network->info->connected_time != 0) {
 		int n = known_network_offset(network->info);
 
+		L_WARN_ON(n < 0);
+
 		if (n >= (int) L_ARRAY_SIZE(rankmod_table))
 			n = L_ARRAY_SIZE(rankmod_table) - 1;
 
