@@ -2,7 +2,7 @@
  *
  *  Wireless daemon for Linux
  *
- *  Copyright (C) 2013-2018  Intel Corporation. All rights reserved.
+ *  Copyright (C) 2013-2019  Intel Corporation. All rights reserved.
  *
  *  This library is free software; you can redistribute it and/or
  *  modify it under the terms of the GNU Lesser General Public
@@ -37,7 +37,9 @@
 
 #include "linux/nl80211.h"
 
+#include "src/missing.h"
 #include "src/iwd.h"
+#include "src/module.h"
 #include "src/ie.h"
 #include "src/crypto.h"
 #include "src/scan.h"
@@ -67,6 +69,7 @@ struct wiphy {
 	uint8_t ext_features[(NUM_NL80211_EXT_FEATURES + 7) / 8];
 	uint8_t max_num_ssids_per_scan;
 	uint32_t max_roc_duration;
+	uint16_t max_scan_ie_len;
 	uint16_t supported_iftypes;
 	uint16_t supported_ciphers;
 	struct scan_freq_set *supported_freqs;
@@ -76,6 +79,7 @@ struct wiphy {
 	struct watchlist state_watches;
 	uint8_t extended_capabilities[EXT_CAP_LEN + 2]; /* max bitmap size + IE header */
 	uint8_t *iftype_extended_capabilities[NUM_NL80211_IFTYPES];
+	uint8_t *supported_rates[NUM_NL80211_BANDS];
 	uint8_t rm_enabled_capabilities[7]; /* 5 size max + header */
 
 	bool support_scheduled_scan:1;
@@ -85,6 +89,8 @@ struct wiphy {
 	bool soft_rfkill : 1;
 	bool hard_rfkill : 1;
 	bool offchannel_tx_ok : 1;
+	bool blacklisted : 1;
+	bool registered : 1;
 };
 
 static struct l_queue *wiphy_list = NULL;
@@ -212,6 +218,9 @@ static void wiphy_free(void *data)
 	for (i = 0; i < NUM_NL80211_IFTYPES; i++)
 		l_free(wiphy->iftype_extended_capabilities[i]);
 
+	for (i = 0; i < NUM_NL80211_BANDS; i++)
+		l_free(wiphy->supported_rates[i]);
+
 	scan_freq_set_free(wiphy->supported_freqs);
 	watchlist_destroy(&wiphy->state_watches);
 	l_free(wiphy->model_str);
@@ -233,6 +242,11 @@ struct wiphy *wiphy_find(int wiphy_id)
 	return l_queue_find(wiphy_list, wiphy_match, L_UINT_TO_PTR(wiphy_id));
 }
 
+bool wiphy_is_blacklisted(const struct wiphy *wiphy)
+{
+	return wiphy->blacklisted;
+}
+
 static bool wiphy_is_managed(const char *phy)
 {
 	char *pattern;
@@ -267,12 +281,20 @@ check_blacklist:
 
 const char *wiphy_get_path(struct wiphy *wiphy)
 {
-	static char path[15];
+	static char path[256];
+
+	L_WARN_ON(snprintf(path, sizeof(path), "%s/%d", IWD_BASE_PATH,
+				wiphy->id) >= (int) sizeof(path));
+	path[sizeof(path) - 1] = '\0';
 
-	snprintf(path, sizeof(path), "/%d", wiphy->id);
 	return path;
 }
 
+uint32_t wiphy_get_id(struct wiphy *wiphy)
+{
+	return wiphy->id;
+}
+
 uint32_t wiphy_get_supported_bands(struct wiphy *wiphy)
 {
 	if (!wiphy->supported_freqs)
@@ -356,6 +378,11 @@ uint8_t wiphy_get_max_num_ssids_per_scan(struct wiphy *wiphy)
 	return wiphy->max_num_ssids_per_scan;
 }
 
+uint16_t wiphy_get_max_scan_ie_len(struct wiphy *wiphy)
+{
+	return wiphy->max_scan_ie_len;
+}
+
 uint32_t wiphy_get_max_roc_duration(struct wiphy *wiphy)
 {
 	return wiphy->max_roc_duration;
@@ -478,6 +505,20 @@ bool wiphy_supports_iftype(struct wiphy *wiphy, uint32_t iftype)
 	return wiphy->supported_iftypes & (1 << (iftype - 1));
 }
 
+const uint8_t *wiphy_get_supported_rates(struct wiphy *wiphy, unsigned int band,
+						unsigned int *out_num)
+{
+	if (band >= L_ARRAY_SIZE(wiphy->supported_rates))
+		return NULL;
+
+	if (out_num)
+		*out_num =
+			(uint8_t *) rawmemchr(wiphy->supported_rates[band], 0) -
+			wiphy->supported_rates[band];
+
+	return wiphy->supported_rates[band];
+}
+
 uint32_t wiphy_state_watch_add(struct wiphy *wiphy,
 				wiphy_state_watch_func_t func,
 				void *user_data, wiphy_destroy_func_t destroy)
@@ -622,20 +663,70 @@ static void parse_supported_frequencies(struct wiphy *wiphy,
 	}
 }
 
+static uint8_t *parse_supported_rates(struct l_genl_attr *attr)
+{
+	uint16_t type;
+	uint16_t len;
+	const void *data;
+	struct l_genl_attr nested;
+	int count = 0;
+	uint8_t *ret;
+
+	if (!l_genl_attr_recurse(attr, &nested))
+		return NULL;
+
+	while (l_genl_attr_next(&nested, NULL, NULL, NULL))
+		count++;
+
+	if (!l_genl_attr_recurse(attr, &nested))
+		return NULL;
+
+	ret = l_malloc(count + 1);
+	ret[count] = 0;
+
+	count = 0;
+
+	while (l_genl_attr_next(&nested, NULL, NULL, NULL)) {
+		struct l_genl_attr nested2;
+
+		if (!l_genl_attr_recurse(&nested, &nested2)) {
+			l_free(ret);
+			return NULL;
+		}
+
+		while (l_genl_attr_next(&nested2, &type, &len, &data)) {
+			if (type != NL80211_BITRATE_ATTR_RATE || len != 4)
+				continue;
+
+			/*
+			 * Convert from the 100kb/s units reported by the
+			 * kernel to the 500kb/s used in 802.11 IEs.
+			 */
+			ret[count++] = *(const uint32_t *) data / 5;
+		}
+	}
+
+	return ret;
+}
+
 static void parse_supported_bands(struct wiphy *wiphy,
 						struct l_genl_attr *bands)
 {
-	uint16_t type, len;
-	const void *data;
+	uint16_t type;
 	struct l_genl_attr attr;
 
 	l_debug("");
 
-	while (l_genl_attr_next(bands, NULL, NULL, NULL)) {
+	while (l_genl_attr_next(bands, &type, NULL, NULL)) {
+		enum nl80211_band band = type;
+
+		if (band != NL80211_BAND_2GHZ && band != NL80211_BAND_5GHZ)
+			continue;
+
 		if (!l_genl_attr_recurse(bands, &attr))
 			continue;
 
-		while (l_genl_attr_next(&attr, &type, &len, &data)) {
+		while (l_genl_attr_next(&attr, &type, NULL, NULL)) {
 			struct l_genl_attr freqs;
 
 			switch (type) {
@@ -645,6 +736,14 @@ static void parse_supported_bands(struct wiphy *wiphy,
 
 				parse_supported_frequencies(wiphy, &freqs);
 				break;
+
+			case NL80211_BAND_ATTR_RATES:
+				if (wiphy->supported_rates[band])
+					continue;
+
+				wiphy->supported_rates[band] =
+					parse_supported_rates(&attr);
+				break;
 			}
 		}
 	}
@@ -755,6 +854,12 @@ static void wiphy_parse_attributes(struct wiphy *wiphy,
 				wiphy->max_num_ssids_per_scan =
 							*((uint8_t *) data);
 			break;
+		case NL80211_ATTR_MAX_SCAN_IE_LEN:
+			if (len != sizeof(uint16_t))
+				l_warn("Invalid MAX_SCAN_IE_LEN attribute");
+			else
+				wiphy->max_scan_ie_len = *((uint16_t *) data);
+			break;
 		case NL80211_ATTR_SUPPORT_IBSS_RSN:
 			wiphy->support_adhoc_rsn = true;
 			break;
@@ -880,37 +985,49 @@ static void wiphy_register(struct wiphy *wiphy)
 		l_info("Unable to add the %s interface to %s",
 				L_DBUS_INTERFACE_PROPERTIES,
 				wiphy_get_path(wiphy));
+
+	wiphy->registered = true;
 }
 
 struct wiphy *wiphy_create(uint32_t wiphy_id, const char *name)
 {
 	struct wiphy *wiphy;
 
-	if (!wiphy_is_managed(name))
-		return NULL;
-
 	wiphy = wiphy_new(wiphy_id);
 	l_strlcpy(wiphy->name, name, sizeof(wiphy->name));
 	l_queue_push_head(wiphy_list, wiphy);
 
-	wiphy_register(wiphy);
+	if (!wiphy_is_managed(name))
+		wiphy->blacklisted = true;
+
 	return wiphy;
 }
 
-void wiphy_update_from_genl(struct wiphy *wiphy, const char *name,
-				struct l_genl_msg *msg)
+void wiphy_update_from_genl(struct wiphy *wiphy, struct l_genl_msg *msg)
 {
+	if (wiphy->blacklisted)
+		return;
+
 	l_debug("");
 
+	wiphy_parse_attributes(wiphy, msg);
+}
+
+void wiphy_update_name(struct wiphy *wiphy, const char *name)
+{
+	bool updated = false;
+
 	if (strncmp(wiphy->name, name, sizeof(wiphy->name))) {
+		l_strlcpy(wiphy->name, name, sizeof(wiphy->name));
+		updated = true;
+	}
+
+	if (updated && wiphy->registered) {
 		struct l_dbus *dbus = dbus_get_bus();
 
-		l_strlcpy(wiphy->name, name, sizeof(wiphy->name));
 		l_dbus_property_changed(dbus, wiphy_get_path(wiphy),
 					IWD_WIPHY_INTERFACE, "Name");
 	}
-
-	wiphy_parse_attributes(wiphy, msg);
 }
 
 static void wiphy_set_station_capability_bits(struct wiphy *wiphy)
@@ -931,7 +1048,7 @@ static void wiphy_set_station_capability_bits(struct wiphy *wiphy)
 
 	ext_capa = wiphy->iftype_extended_capabilities[NL80211_IFTYPE_STATION];
 
-	if (!l_settings_get_bool(iwd_get_config(), "General", "disable_anqp",
+	if (!l_settings_get_bool(iwd_get_config(), "General", "DisableANQP",
 				&anqp_disabled))
 		anqp_disabled = true;
 
@@ -958,6 +1075,8 @@ static void wiphy_setup_rm_enabled_capabilities(struct wiphy *wiphy)
 
 	wiphy->rm_enabled_capabilities[0] = IE_TYPE_RM_ENABLED_CAPABILITIES;
 	wiphy->rm_enabled_capabilities[1] = 5;
+	/* Bits: Passive (4), Active (5), and Beacon Table (6) capabilities */
+	wiphy->rm_enabled_capabilities[2] = 0x70;
 
 	/*
 	 * TODO: Support at least Link Measurement if TX_POWER_INSERTION is
@@ -967,6 +1086,8 @@ static void wiphy_setup_rm_enabled_capabilities(struct wiphy *wiphy)
 
 void wiphy_create_complete(struct wiphy *wiphy)
 {
+	wiphy_register(wiphy);
+
 	if (util_mem_is_zero(wiphy->permanent_addr, 6)) {
 		int err = wiphy_get_permanent_addr_from_sysfs(wiphy);
 
@@ -988,7 +1109,8 @@ bool wiphy_destroy(struct wiphy *wiphy)
 	if (!l_queue_remove(wiphy_list, wiphy))
 		return false;
 
-	l_dbus_unregister_object(dbus_get_bus(), wiphy_get_path(wiphy));
+	if (wiphy->registered)
+		l_dbus_unregister_object(dbus_get_bus(), wiphy_get_path(wiphy));
 
 	wiphy_free(wiphy);
 	return true;
@@ -1166,16 +1288,12 @@ static int wiphy_init(void)
 {
 	struct l_genl *genl = iwd_get_genl();
 	const struct l_settings *config = iwd_get_config();
-	const char *s = l_settings_get_value(config, "General",
-							"mac_randomize_bytes");
 	const char *whitelist = iwd_get_phy_whitelist();
 	const char *blacklist = iwd_get_phy_blacklist();
+	const char *s;
 
 	nl80211 = l_genl_family_new(genl, NL80211_GENL_NAME);
 
-	if (s && !strcmp(s, "nic"))
-		mac_randomize_bytes = 3;
-
 	/*
 	 * This is an extra sanity check so that no memory is leaked
 	 * in case the generic netlink handling gets confused.
@@ -1204,6 +1322,18 @@ static int wiphy_init(void)
 	if (blacklist)
 		blacklist_filter = l_strsplit(blacklist, ',');
 
+	s = l_settings_get_value(config, "General",
+						"AddressRandomizationRange");
+	if (s) {
+		if (!strcmp(s, "nic"))
+			mac_randomize_bytes = 3;
+		else if (!strcmp(s, "full"))
+			mac_randomize_bytes = 6;
+		else
+			l_warn("Invalid [General].AddressRandomizationRange"
+				" value: %s", s);
+	}
+
 	return 0;
 }
 
